/*
File name: burn_tree_pm.java
Tag: ACCEPTED
Author: pkunyavskiy
Change time: Thu Mar 30 02:29:01 MSK 2017

*/
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;
import java.io.*;


/**
 * @author Pavel Mavrin
 */
public class burn_tree_pm {

    class Node {
        List<Edge> edges = new ArrayList<>();
        List<Edge> children = new ArrayList<>();

        int id, t;
        int downL, downR;
        int upL, upR;

        int[] minPref;
        int[] minSuf;
        int[] maxPref;
        int[] maxSuf;

        public Node(int id, int t) {
            this.id = id;
            this.t = t;
        }
    }

    class Edge {
        Node dst;
        int l, r;

        public Edge(Node dst, int l, int r) {
            this.dst = dst;
            this.l = l;
            this.r = r;
        }
    }


    private void solve() throws IOException {
        int n = nextInt();
        Node[] nodes = new Node[n];
        for (int i = 0; i < n; i++) {
            nodes[i] = new Node(i, nextInt());
        }
        for (int i = 0; i < n - 1; i++) {
//            int v = nextInt() - 1;
//            int u = nextInt() - 1;
            int l = nextInt();
            int r = nextInt();
            nodes[i].edges.add(new Edge(nodes[i + 1], l, r));
            nodes[i + 1].edges.add(new Edge(nodes[i], l, r));
        }
        dfsDown(nodes[0], null);
        dfsUp(nodes[0], 0, Integer.MAX_VALUE);
//        for (int i = 0; i < n; i++) {
//            System.out.println(nodes[i].downL + " " + nodes[i].downR + " " + nodes[i].upL + " " + nodes[i].upR);
//            System.out.println(Arrays.toString(nodes[i].minPref));
//            System.out.println(Arrays.toString(nodes[i].minSuf));
//            System.out.println(Arrays.toString(nodes[i].maxPref));
//            System.out.println(Arrays.toString(nodes[i].maxSuf));
//        }
        for (int i = 0; i < n; i++) {
            int ll = Math.max(nodes[i].downL, nodes[i].upL);
            int rr = Math.min(nodes[i].downR, nodes[i].upR);
            if (ll > rr) {
                out.println(-1);
            } else {
                out.println(ll);
            }
        }
    }


    private void dfsDown(Node x, Node parent) {
        for (Edge edge : x.edges) {
            Node y = edge.dst;
            if (y == parent) continue;
            x.children.add(edge);
            dfsDown(y, x);
        }
        int n = x.children.size();
        x.minPref = new int[n + 1];
        x.minSuf = new int[n + 1];
        x.minPref[0] = x.minSuf[n] = Integer.MAX_VALUE;
        x.maxPref = new int[n + 1];
        x.maxSuf= new int[n + 1];
        x.maxPref[0] = x.maxSuf[n] = 0;
        List<Edge> children = x.children;
        for (int i = 0; i < n; i++) {
            Edge edge = children.get(i);
            Node y = edge.dst;
            int ll = Math.max(edge.l, y.downL);
            int rr = Math.min(edge.r, y.downR);
            if (ll > rr) {
                ll = Integer.MAX_VALUE;
                rr = 0;
            } else {
                if (ll == edge.l)
                    ll -= x.t;
            }
            x.minPref[i + 1] = x.minSuf[i] = rr;
            x.maxPref[i + 1] = x.maxSuf[i] = ll;
        }
        for (int i = 0; i < n; i++) {
            x.minPref[i + 1] = Math.min(x.minPref[i], x.minPref[i + 1]);
            x.maxPref[i + 1] = Math.max(x.maxPref[i], x.maxPref[i + 1]);
        }
        for (int i = n - 1; i >= 0; i--) {
            x.minSuf[i] = Math.min(x.minSuf[i], x.minSuf[i + 1]);
            x.maxSuf[i] = Math.max(x.maxSuf[i], x.maxSuf[i + 1]);
        }
        x.downL = x.maxSuf[0];
        x.downR = x.minSuf[0];
    }

    private void dfsUp(Node x, int l, int r) {
        x.upL = l;
        x.upR = r;
        List<Edge> children = x.children;
        int n = children.size();
        for (int i = 0; i < n; i++) {
            Edge edge = children.get(i);
            Node y = edge.dst;
            int ll = Math.max(x.maxPref[i], x.maxSuf[i + 1]);
            int rr = Math.min(x.minPref[i], x.minSuf[i + 1]);
            ll = Math.max(ll, l);
            rr = Math.min(rr, r);
            ll = Math.max(ll, edge.l);
            rr = Math.min(rr, edge.r);
            if (ll > rr) {
                ll = Integer.MAX_VALUE;
                rr = 0;
            } else {
                if (ll == edge.l)
                    ll -= y.t;
            }
            dfsUp(y, ll, rr);
        }
    }

    // ------------------

    BufferedReader br;
    StringTokenizer st;
    PrintWriter out;
    {
        try {
            br = new BufferedReader(new FileReader("servers.in"));
            out = new PrintWriter("servers.out");
        } catch (IOException ignored) {
        }
    }

    String next() throws IOException {
        while (st == null || !st.hasMoreTokens()) {
            st = new StringTokenizer(br.readLine());
        }
        return st.nextToken();
    }

    int nextInt() throws IOException {
        return Integer.parseInt(next());
    }

    public static void main(String[] args) throws IOException {
        new burn_tree_pm().run();
    }

    private void run() throws IOException {
        solve();
        out.close();
    }

}

