/**
 * Sergey Kopeliovich (burunduk30@gmail.com)
 * Time = O(nA*logA)
 * Memory = O(A)
 * idea = backward-DP with A states, use range tree 
 */

#include <climits>
#include <cassert>
#include <fstream>
#include <vector>
#include <functional>

using namespace std;

#define forn(i, n) for (int i = 0; i < (int)(n); i++)

const int N = 3e6 + 17, K = 1000;
const long long MUL = 1000 * 1000 * 1000;
int n, A, x[K], y[K], c[K];
long long f[2 * N];
void finish( int a ) {
	for (a = (a + N) / 2; a > 0; a >>= 1)
		f[a] = min(f[2 * a], f[2 * a + 1]);
}

long long get( int l, int r ) {
	long long res = (long long)2e18;
	for (l += N, r += N; l < r; l >>= 1, r >>= 1) {
		if (l & 1) res = min(res, f[l++]);
		if (r & 1) res = min(res, f[--r]);
	}
	return res;
}

int main() {
	ifstream in("anti.in");
	ofstream out("anti.out");

	in >> n >> A;
	assert(n <= K && A < N);
	forn(i, n)
		in >> x[i] >> y[i] >> c[i];
	f[N + A] = 1LL * A * MUL, finish(A);
	for (int a = A - 1; a >= 0; a--) {
		long long &r = f[N + a];
		r = 1LL * a * MUL;
		forn(i, n)
			if (a + y[i] <= A)
			{
				long long tmp = get(a + x[i], a + y[i] + 1);
				tmp -= c[i];
				r = max(r, tmp);
			}
		finish(a);
	}
	out << f[N] << endl;
	out.close();
}
