/*
File name: segments_ma_nlogn_clev.cpp
Tag: MAIN
Author: niyaznigmatul
Change time: Wed Mar 29 22:14:01 MSK 2017

*/
#include <cstdio>
#include <cassert>
#include <tuple>
#include <algorithm>
#include <numeric>
#include <vector>
using namespace std;

typedef long long llong;

#define eprintf(...) fprintf(stderr, __VA_ARGS__)

#ifndef LOCAL42
    #define TASK "internship"
#endif

vector<pair<int, int>> pts;

const int N = 1000500;
const llong inf = 1e16;

vector<int> A, WA;
vector<llong> SA;
vector<int> B, WB;
vector<llong> SB;

void read(int n, vector<int>& X, vector<int>& WX, vector<llong>& SX) {
    X.resize(n);
    WX.resize(n);
    SX.resize(n + 1);
    for (int i = 0; i < n; i++) {
        scanf("%d", &X[i]);
        --X[i];
    }
    for (int i = 0; i < n; i++) {
        scanf("%d", &WX[i]);
        SX[i + 1] = SX[i] + WX[i];
    }
}

struct cake {
    vector<llong> T, D;
    int TN;
    cake() {
        TN = 1;
        while (TN < (int)B.size() + 1)
            TN <<= 1;
        T.resize(2 * TN, -inf);
        D.resize(2 * TN, 0);
        for (int i = 0; i <= (int)B.size(); i++) {
            T[TN + i] += -SB[i];
        }
        for (int i = TN - 1; i > 0; i--) {
            T[i] = max(T[2 * i], T[2 * i + 1]);
        }
    }
    pair<llong, int> getMax() {
        llong mx = T[1] + D[1];
        int v = 1;
        while (v < TN) {
            if (T[2 * v] + D[2 * v] > T[2 * v + 1] + D[2 * v + 1]) {
                v <<= 1;
            } else {
                v <<= 1;
                ++v;
            }
        }
        return make_pair(mx, v - TN);
    }

    void add(int l, int r, int L, int R, int v, llong x) {
        if (r < L || l > R) {
            return;
        } else if (l <= L && R <= r) {
            D[v] += x;
        } else {
            add(l, r, L, (L + R) / 2, 2 * v, x);
            add(l, r, (L + R) / 2 + 1, R, 2 * v + 1, x);
            T[v] = max(T[2 * v] + D[2 * v], T[2 * v + 1] + D[2 * v + 1]);
        }
    }
    void add(int l, int r, llong x) {
        add(l, r, 0, TN - 1, 1, x);
    }
};

struct conqueror {
    vector<pair<int, llong>> SL = {{-1, 0ll}};
    vector<pair<int, llong>> SR = {{-1, 0ll}};

    int mid;
    int last = -1;
    tuple<llong, int, int> ans;

    cake C;

    conqueror(int _mid) : mid(_mid)
    { }

    void feed(int x, int y, bool relaxOnly = false) {
        C.add(last + 1, y, inf + SA.back());
        auto cur = C.getMax();
        cur.first += SB[y];
        if (get<0>(ans) < cur.first) {
            ans = make_tuple(cur.first, cur.second, y - 1);
        }

        if (relaxOnly) {
            return;
        }

        auto& S = (x < mid) ? SL : SR;
        S.emplace_back(y, (x < mid) ? SA[mid] : SA.back() - SA[mid]);
        llong w = (x < mid) ? (SA[mid] - SA[x + 1]) : (SA[x] - SA[mid]);
        while (S.back().second > w) {
            assert(S.size() >= 2);
            int from = S[(int)S.size() - 2].first + 1;
            int to = S[(int)S.size() - 1].first;
            C.add(from, to, -S.back().second + w);
            S.pop_back();
        }
        S.emplace_back(y, w);
        last = y;
    }
};

tuple<llong, int, int, int, int> restoreByB(int l, int r) {
    vector<int> usedB(B.begin() + l, B.begin() + r + 1);
    sort(usedB.begin(), usedB.end());
    tuple<int, int, int> best(0, 0, -1);
    int last = -1;
    for (int i = 0; i < (int)A.size(); i++) {
        if (binary_search(usedB.begin(), usedB.end(), A[i])) {
            last = -1;
            continue;
        }
        if (last == -1) {
            last = i;
        }
        if (get<0>(best) < i - last + 1) {
            best = make_tuple(i - last + 1, last, i);
        }
    }
    int al, ar;
    tie(ignore, al, ar) = best;
    return make_tuple(SA[ar + 1] - SA[al] + SB[r + 1] - SB[l], al, ar, l, r);
}

tuple<llong, int, int, int, int> solve() {
    llong tothorW = accumulate(WA.begin(), WA.end(), 0ll);
    int mid = 0;
    llong leftW = 0;
    while (leftW * 2 < tothorW) {
        leftW += WA[mid++];
    }
    conqueror C(mid);
    sort(pts.begin(), pts.end(), [] (pair<int, int> a, pair<int, int> b) { return a.second < b.second; });
    for (const auto& pt : pts) {
        C.feed(pt.first, pt.second);
    }
    C.feed(-42, B.size(), true /* relaxOnly */);
    auto resB = C.ans;
    auto res = restoreByB(get<1>(resB), get<2>(resB));
    //assert(get<0>(res) == get<0>(resB));
    return res;
}

int main() {
#ifdef TASK
    freopen(TASK ".in", "r", stdin);
    freopen(TASK ".out", "w", stdout);
#endif
    int n, m;
    scanf("%d %d", &n, &m);
    pts.resize(n + m, {-1, -1});
    read(n, A, WA, SA);
    read(m, B, WB, SB);
    for (int i = 0; i < (int)A.size(); i++) {
        pts[A[i]].first = i;
    }
    for (int i = 0; i < (int)B.size(); i++) {
        pts[B[i]].second = i;
    }
    pts.erase(remove_if(pts.begin(), pts.end(), [] (pair<int, int> pt) { return pt.first == -1 || pt.second == -1; }), pts.end());
    auto ansv = solve();
    A.swap(B);
    WA.swap(WB);
    SA.swap(SB);
    for (auto& pt : pts) {
        swap(pt.first, pt.second);
    }
    auto ansh = solve();
    swap(get<1>(ansh), get<3>(ansh));
    swap(get<2>(ansh), get<4>(ansh));
    eprintf("ansv = (%lld, %d, %d, %d, %d)\n", get<0>(ansv), get<1>(ansv), get<2>(ansv), get<3>(ansv), get<4>(ansv));
    eprintf("ansh = (%lld, %d, %d, %d, %d)\n", get<0>(ansh), get<1>(ansh), get<2>(ansh), get<3>(ansh), get<4>(ansh));

    {
        auto ans = max(ansv, ansh);
        llong totw;
        int l0, r0, l1, r1;
        tie(totw, l0, r0, l1, r1) = ans;
        l0++, r0++, l1++, r1++;
        if (r0 == 0)
            l0 = 0;
        if (r1 == 0)
            l1 = 0;
        printf("%lld\n%d %d\n%d %d\n", totw, l0, r0, l1, r1);
    }
}

