#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <inttypes.h>
#include <vector>
#include <random>
#include <algorithm>
#include <functional>

const int N = 1<<19;

struct dsu_t {
    const int n;
    int *base;
    char *used;
    int *link;
    int *sz;
    int64_t *sum;
    int *ls;
    int64_t mx_sum;
    int mx_sum_i;

    inline void upd_mx_sum(int i) {
        if (sum[i] > mx_sum)
            mx_sum = sum[i],
            mx_sum_i = i;
    }

    void clear() {
        for (int i = 0; i < n; ++i)
            used[i] = false,
            link[i] = i,
            sz[i] = 1,
            sum[i] = 0,
            ls[i] = i,
            mx_sum = 0,
            mx_sum_i = -1;
    }

    int get(int i) {
        return link[i] == i ? i : link[i] = get(link[i]);
    }

    void unite(int a, int b) {
        a = get(a);
        b = get(b);
        if (a == b)
            return;
        if (sz[a] > sz[b])
            std::swap(a, b);
        link[a] = b;
        sz[b] += sz[a];
        sum[b] += sum[a];
        ls[b] = std::min(ls[b], ls[a]);
        upd_mx_sum(b);
    }

    void add(int i) {
        assert(!used[i]);
        used[i] = true;
        sum[i] = base[i];
        if (i && used[i-1]) unite(i-1, i);
        if (i+1 < n && used[i+1]) unite(i, i+1);
    }

    void get_mx_sum(int64_t& res, int& l, int& r) {
        res = mx_sum;
        if (mx_sum_i == -1)
            l = -1,
            r = 0;
        else
            l = ls[mx_sum_i],
            r = l + sz[mx_sum_i];
    }

    dsu_t(int _n, int* _base) :
        n(_n),
        base(_base),
        used(new char[_n]),
        link(new int[_n]),
        sz(new int[_n]),
        sum(new int64_t[_n]),
        ls(new int[_n]) {};
};

int vals1[N];
int costs1[N];
int vals2[N];
int costs2[N];

char cnt[2*N];
std::vector<int> poss[2*N];

int main()
{
    int n, m; scanf("%d%d", &n, &m);
    for (int i = 0; i < n; ++i)
        scanf("%d", vals1 + i);
    for (int i = 0; i < n; ++i)
        scanf("%d", costs1 + i);
    for (int i = 0; i < m; ++i)
        scanf("%d", vals2 + i);
    for (int i = 0; i < m; ++i)
        scanf("%d", costs2 + i);

    for (int i = 0; i < m; ++i)
        poss[vals2[i]].push_back(i);

    dsu_t dsu(m, costs2);

    int64_t best_sum = std::accumulate(costs2, costs2+m, 0LL);
    int bl1 = -1;
    int br1 =  0;
    int bl2 = 0;
    int br2 = m;

    for (int l = 0; l < n; ++l) {
        dsu.clear();
        for (int i = 1; i <= m+n; ++i)
            cnt[i] = false;

        int64_t sum = 0;
        int r = l;
        while (r < n && !cnt[vals1[r]])
            sum += costs1[r],
            cnt[vals1[r]] = true,
            r++;

        for (int i = 1; i <= m+n; ++i) if (!cnt[i])
            for (int p : poss[i])
                dsu.add(p);

        while (l < r) {
            int64_t sum2;
            int l2, r2;
            dsu.get_mx_sum(sum2, l2, r2);
            if (sum + sum2 > best_sum)
                best_sum = sum + sum2,
                bl1 = l,
                br1 = r,
                bl2 = l2,
                br2 = r2;
            --r;
            sum -= costs1[r];
            cnt[vals1[r]] = false;
            for (int p : poss[vals1[r]])
                dsu.add(p);
        }
    }

    printf("%lld\n", best_sum);
    printf("%d %d\n", bl1+1, br1);
    printf("%d %d\n", bl2+1, br2);
    return 0;
}
