#include <algorithm>
#include <iostream>
#include <bitset>
#include <set>
#include <map>
#include <string>
#include <cassert>
#include <ctime>
#include <random>

using namespace std;

template<class T> inline void chkmin(T& a, T b) { if (a > b) a = b; }
template<class T> inline void chkmax(T& a, T b) { if (a < b) a = b; }

typedef long long ll;
typedef long double ld;

struct Point {
	ll x, y;

	Point() : x(0), y(0) {}
	Point(ll x, ll y) : x(x), y(y) {}
};

ll operator* (Point a, Point b) {
	return a.x * b.x + a.y * b.y;
}

ll operator% (Point a, Point b) {
	return a.x * b.y - a.y * b.x;
}

bool operator< (Point a, Point b) {
	return make_pair(a.x, a.y) < make_pair(b.x, b.y);
}

Point operator- (Point a, Point b) {
	return Point(a.x - b.x, a.y - b.y);
}

bool operator== (Point a, Point b) {
	return a.x == b.x && a.y == b.y;
}

ostream& operator<< (ostream& fin, Point p) {
	return fin << p.x << ' ' << p.y;
}

vector<Point> toPoint;

bool LessNaive(int a, int b) {
	return toPoint[a] < toPoint[b];
}

Point center;

bool Less(int q, int w) {
	Point a = toPoint[q];
	Point b = toPoint[w];
	a = a - center;
	b = b - center;
	assert(a % b != 0);
	return a % b > 0;
}

bool Ask(vector<int> pol) {
	cout << "? " << pol.size() << ' ';
	for (int p : pol) {
		cout << p + 1 << ' ';
	}
	cout << '\n';
	string response;
	cin >> response;
	return response == "Yes";
}

vector<int> ConvexHull(vector<int> pol) {
	int n = pol.size();
	sort(pol.begin(), pol.end(), LessNaive);
	center = toPoint[pol[0]];
	sort(pol.begin() + 1, pol.end(), Less);
	vector<int> res(1, pol[0]);
	for (int i = 1; i < n; ++i) {
		while (res.size() >= 2 && (toPoint[pol[i]] - toPoint[res.back()]) % (toPoint[res.back()] - toPoint[res[res.size() - 2]]) > 0)
			res.pop_back();
		res.push_back(pol[i]);
	}
	return res;
}

void Catch(vector<int> pol) {
	int n = pol.size();
	sort(pol.begin(), pol.end(), LessNaive);
	center = toPoint[pol[0]];
	sort(pol.begin() + 1, pol.end(), Less);
	vector<int> res(1, 0);
	for (int i = 1; i < n; ++i) {
		while (res.size() >= 2 && (toPoint[pol[i]] - toPoint[pol[res.back()]]) % (toPoint[pol[res.back()]] - toPoint[pol[res[res.size() - 2]]]) > 0)
			res.pop_back();
		res.push_back(i);
	}
	int l = 1, r = res.size() - 1;
	while (r - l > 1) {
		int m1 = (l + r) / 2;
		vector<int> toAsk;
		for (int i = 0; i <= m1; ++i)
			toAsk.push_back(pol[res[i]]);
		if (Ask(toAsk))
			r = m1;
		else
			l = m1;
	}
	vector<int> nextBound(pol.begin() + res[l], pol.begin() + res[r] + 1);
	if (nextBound.size() >= 3 && Ask(ConvexHull(nextBound))) {
		Catch(nextBound);
		return;
	}
	reverse(nextBound.begin(), nextBound.end());
	int n1 = nextBound.size();
	for (int i = 0; i < n1; ++i)
		nextBound.push_back(nextBound[i]);
	int shift = 0;
	for (; shift < n1; ++shift)
		if (nextBound[shift] == pol[res[r]])
			break;
	l = shift, r = n1 + shift;
	while (r - l > 1) {
		int m = (l + r) / 2;
		vector<int> toAsk;
		toAsk.push_back(pol[0]);
		toAsk.push_back(nextBound[m]);
		toAsk.push_back(nextBound[shift]);
		if (Ask(toAsk))
			r = m;
		else
			l = m;
	}
	cout << "! " << 3 << ' ' << pol[0] + 1 << ' ' << nextBound[r] + 1 << ' ' << nextBound[l] + 1 << '\n';
}

int main() {
	int n;
	cin >> n;
	toPoint.resize(n);
	for (int i = 0; i < n; ++i)
		cin >> toPoint[i].x >> toPoint[i].y;
	vector<int> neut;
	for (int i = 0; i < n; ++i)
		neut.push_back(i);
	int q;
	cin >> q;
	for (int i = 0; i < q; ++i)
		Catch(neut);
	//while (true) ++n;
	//cout << n << '\n';
}

