// ConsoleApplication1.cpp : Defines the entry point for the console application.
//

#include <iostream>
#include <vector>
#include <string>
#include <cmath>
#include <algorithm>
#include <cstdio>
#include <ctime>
#include <cstdlib>
#include <map>

using namespace std;

#define int long long

struct Point {
	int x, y;
	int num;
	Point() : x(0), y(0), num(0) {}
	Point(int xy, int yx) : x(xy), y(yx), num(0) {}

	bool operator<(const Point &P) const {
		return y < P.y || (y == P.y && x < P.x);
	}

	double get_polar(Point p) {
		int xv = x - p.x;
		int vy = y - p.y;
		double d = atan2(vy, xv);

		if (d < 0)
			d += (double)2 * 3.1415;
		return d;
	}
};

int get_vector_mul(Point a, Point b) {
	return a.x * b.y - a.y * b.x;
}

auto xcmp = [](Point a, Point b) { return a.x < b.x;  };
auto ycmp = [](Point a, Point b) { return a.y < b.y;  };

const int p = 37037;

int vhash(vector<Point> pts) {
	int res = 0;
	int cp = p;
	for (auto it : pts) {
		res += it.x * cp + it.y * cp * cp;
		cp *= p;
	}
	return res;
}

map<int, pair<bool, vector<Point>>> precount;

vector<Point> build_hull(vector<Point> pts, Point &def) {

	if (precount[vhash(pts)].first)
		return precount[vhash(pts)].second;

	sort(pts.begin(), pts.end());

	auto acmp = [&pts](Point a, Point b) {
		return a.get_polar(pts.front()) < b.get_polar(pts.front());
	};

	stable_sort(pts.begin() + 1, pts.end(), acmp);
	vector<Point> hull;
	hull.push_back(pts[0]);
	hull.push_back(pts[1]);

	for (int i = 2; i < pts.size(); i++) {
		Point a = pts[i];
		Point b = hull[hull.size() - 1];
		Point c = hull[hull.size() - 2];
		while (hull.size() > 2 && get_vector_mul(Point(a.x - b.x, a.y - b.y), Point(b.x - c.x, b.y - c.y)) > 0) {
			def = hull.back();
			hull.pop_back();
		}
		hull.push_back(pts[i]);
	}
	precount[vhash(pts)].first = 1;
	return precount[vhash(pts)].second = hull;

}

bool ask(vector<Point> hull) {
	cout << "? " << hull.size() << ' ';
	for (auto it : hull) {
		cout << it.num + 1 << ' ';
	}
	cout << endl;
	string s;
	cin >> s;
	return s[0] == 'Y';

}

void ans(vector<Point> hull) {
	cout << "! " << hull.size() << ' ';
	for (auto it : hull)
		cout << it.num + 1 << ' ';
	cout << endl;
}

void solve(vector<Point> trans) {
	Point tmp;
	if (trans.size() < 4) {
		ans(trans);
		return;
	}
	Point P2;
	vector<Point> hl = build_hull(trans, P2);

	Point P1 = hl[rand() % hl.size()];

	vector<Point> top;
	vector<Point> bot;

	top.push_back(P1);
	top.push_back(P2);
	bot.push_back(P1);
	bot.push_back(P2);

	for (int i = 0; i < trans.size(); i++) {
		Point p = trans[i];
		if (p.num != P1.num && p.num != P2.num) {
			if (get_vector_mul(Point(P2.x - P1.x, P2.y - P1.y), Point(p.x - P2.x, p.y - P2.y)) > 0)
				bot.push_back(p);
			else
				top.push_back(p);
		}
	}

	if (top.size() < 3) {
		solve(trans);
		return;
	}

	if (bot.size() < 3) {
		solve(trans);
		return;
	}

	vector<Point> tophull = build_hull(top, tmp);
	vector<Point> bothull = build_hull(bot, tmp);

	if (tophull.size() > 2 && ask(tophull)) {
		if (tophull.size() == top.size())
			ans(tophull);
		else
			solve(top);
	}
	else {
		if (bothull.size() > 2 && bothull.size() == bot.size())
			ans(bothull);
		else
			solve(bot);
	}

}

signed main()
{
	srand(time(nullptr));
	int n;
	cin >> n;

	vector<Point> trans(n);

	for (int i = 0; i < n; i++) {
		int x, y;
		cin >> x >> y;
		trans[i] = Point(x, y);
		trans[i].num = i;
	}

	int q;
	cin >> q;
	for (int i = 0; i < q; i++) {
		solve(trans);
	}

	return 0;
}