// ConsoleApplication1.cpp : Defines the entry point for the console application.
//

#include <iostream>
#include <vector>
#include <string>
#include <cmath>
#include <algorithm>
#include <cstdio>
#include <ctime>
#include <cstdlib>
#include <map>
#include <cassert>

using namespace std;

#define int long long

struct Point {
	int x, y;
	short num;
	Point() : x(0), y(0), num(0) {}
	Point(int xy, int yx) : x(xy), y(yx), num(0) {}

	bool operator<(const Point &P) const {
		return y < P.y || (y == P.y && x < P.x);
	}

	double get_polar(Point p) {
		int xv = x - p.x;
		int vy = y - p.y;
		double d = atan2(vy, xv);

		if (d < 0)
			d += (double)2 * 3.1415926;
		return d;
	}
};

int get_vector_mul(Point a, Point b) {
	return a.x * b.y - a.y * b.x;
}

vector<Point> build_hull(vector<Point> pts) {

	sort(pts.begin(), pts.end());

	auto acmp = [&pts](Point a, Point b) {
		return a.get_polar(pts.front()) < b.get_polar(pts.front());
	};

	stable_sort(pts.begin() + 1, pts.end(), acmp);
	vector<Point> hull;
	hull.push_back(pts[0]);
	hull.push_back(pts[1]);

	for (int i = 2; i < pts.size(); i++) {
		Point a = pts[i];
		Point b = hull[hull.size() - 1];
		Point c = hull[hull.size() - 2];
		while (hull.size() > 2 && get_vector_mul(Point(b.x - c.x, b.y - c.y), Point(a.x - b.x, a.y - b.y)) < 0) {
			hull.pop_back();
		}
		hull.push_back(pts[i]);
	}
	return hull;

}

bool ask(vector<Point> hull) {
	cout << "? " << hull.size() << ' ';
	for (auto it : hull) {
		cout << it.num + 1 << ' ';
	}
	cout << endl;
	string s;
	cin >> s;
	return s[0] == 'Y';

}

void ans(vector<Point> hull) {
	cout << "! " << hull.size() << ' ';
	for (auto it : hull)
		cout << it.num + 1 << ' ';
	cout << endl;
}

void solve(vector<Point> trans) {
	tops:
	vector<Point> hl = build_hull(trans);

	if (hl.size() == trans.size()) {
		ans(trans);
		return;
	}

	Point P1 = hl[rand() % hl.size()];
	Point P2 = hl[rand() % hl.size()];

	vector<Point> top;
	vector<Point> bot;

	top.push_back(P1);
	top.push_back(P2);
	bot.push_back(P1);
	bot.push_back(P2);

	for (int i = 0; i < trans.size(); i++) {
		Point p = trans[i];
		if (p.num != P1.num && p.num != P2.num) {
			if (get_vector_mul(Point(P2.x - P1.x, P2.y - P1.y), Point(p.x - P2.x, p.y - P2.y)) > 0)
				bot.push_back(p);
			else
				top.push_back(p);
		}
	}


	vector<Point> tophull = build_hull(top);
	vector<Point> bothull = build_hull(bot);

	if (top.size() > 2 && ask(tophull)) {
		if (tophull.size() == top.size())
			ans(tophull);
		else {
			trans = top;
			goto tops;
		}
		return;
	}
	else {
		if (bot.size() > 2 && bothull.size() == bot.size())
			ans(bothull);
		else if (bot.size() > 2) {
			trans = bot;
			goto tops;
		}
		return;
	}
	assert(false);
}

signed main()
{
	srand(time(nullptr));
	int n;
	cin >> n;

	vector<Point> trans(n);

	for (int i = 0; i < n; i++) {
		int x, y;
		cin >> x >> y;
		trans[i] = Point(x, y);
		trans[i].num = i;
	}

	int q;
	cin >> q;
	for (int i = 0; i < q; i++) {
		solve(trans);
	}

	return 0;
}