#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

struct point {
    int x, y, id;

    point(int x, int y, int id): x(x), y(y), id(id) {}
    point(int x, int y): x(x), y(y), id(0) {}
    point(): x(0), y(0), id(0) {}

    point(const point &a, const point &b) {
        x = b.x - a.x;
        y = b.y - a.y;
    }
};

point operator-(const point &a, const point &b) {
    return point(a.x - b.x, a.y - b.y);
}

long long operator%(const point &a, const point &b) {
    return a.x * b.y - b.x * a.y;
}

typedef vector<point> poly;

point good(0, 0);
bool compare(const point &a, const point &b) {
    point p1(good, a), p2(good, b);
    if (p1.x == p1.y && p1.y == 0)
        return false;
    if (p2.x == p2.y && p2.y == 0)
        return true;
    return p1 % p2 > 0;
}


poly algo(vector<point> points) {
    int n = points.size();
    good = points[0];
    for (int i = 1; i < n; ++i) {
        if (points[i].x < good.x || (points[i].x == good.x && points[i].y < good.y))
            good = points[i];
        sort(points.begin(), points.end(), compare);
    }

    poly result;
    result.push_back(points[0]);
    int size = 1;
    for (int i = 1; i < n; i++) {
        while (size >= 2) {
            point v1(result[size-2], result[size-1]),
                  v2(result[size-1], points[i]);
            if (v1 % v2 > 0)
                break;
            result.pop_back();
            --size;
        }
        result.push_back(points[i]);
        ++size;
    }
    return result;
}

// 1 = Yes, -1 = No, 0 = Answer found
int check(vector<point>& polygon) {
    int init_size = polygon.size();
    if (init_size < 3) return -1;
    poly new_polygon = algo(polygon);
    int size = new_polygon.size();
    cout << "? " << size << ' ';
    for (auto& pnt: new_polygon) {
        cout << pnt.id + 1 << ' ';
    }
    cout << endl;
    string ans;
    cin >> ans;
    if (ans == "Yes") {
        if (size == init_size) {
            cout << "! " << size << ' ';
            for (auto& pnt: new_polygon) {
                cout << pnt.id + 1 << ' ';
            }
            cout << endl;
            return 0;
        } else
            return 1;
    } else
        return -1;
}

int main() {
    int n;
    cin >> n;
    vector<point> p(n);
    for (int i = 0; i < n; ++i) {
        cin >> p[i].x >> p[i].y;
        p[i].id = i;
    }
    int q;
    cin >> q;
    string ans;
    int left, right, size_full;
    while (q--) {
        // binary search for X_min
        left = -1e9, right = 1e9;
        while (left < right) {
            int middle = (left + right) / 2 + (left + right) % 2;
            vector<point> p_temp;
            for (auto &pnt: p) {
                if (pnt.x >= middle)
                    p_temp.push_back(pnt);
            }
            int result = check(p_temp);
            if (result == 0)
                return 0;
            if (result > 0)
                left = middle;
            else
                right = middle - 1;
        }
        int xmin = left;


        // binary search for Y_min
        left = -1e9, right = 1e9;
        while (left < right) {
            int middle = (left + right) / 2 + (left + right) % 2;
            vector<point> p_temp;
            for (auto &pnt: p) {
                if (pnt.y >= middle)
                    p_temp.push_back(pnt);
            }
            int result = check(p_temp);
            if (result == 0)
                return 0;
            if (result > 0)
                left = middle;
            else
                right = middle - 1;
        }
        int ymin = left;


        // binary search for X_max
        left = -1e9, right = 1e9;
        while (left < right) {
            int middle = (left + right) / 2;
            vector<point> p_temp;
            for (auto &pnt: p) {
                if (pnt.x <= middle)
                    p_temp.push_back(pnt);
            }
            int result = check(p_temp);
            if (result == 0)
                return 0;
            if (result > 0)
                right = middle;
            else
                left = middle + 1;
        }
        int xmax = left;


        // binary search for Y_max
        left = -1e9, right = 1e9;
        while (left < right) {
            int middle = (left + right) / 2;
            vector<point> p_temp;
            for (auto &pnt: p) {
                if (pnt.y <= middle)
                    p_temp.push_back(pnt);
            }
            int result = check(p_temp);
            if (result == 0)
                return 0;
            if (result > 0)
                right = middle;
            else
                left = middle + 1;
        }
        int ymax = left;

        vector<point> p_temp;
        for (auto &pnt: p) {
            if (pnt.x >= xmin && pnt.x <= xmax && pnt.y >= ymin && pnt.y <= ymax)
                p_temp.push_back(pnt);
        }

        p_temp = algo(p_temp);
        int len = p_temp.size();
        cout << "! " << len << ' ';
        for (int i = 0; i < len; ++i) {
            cout << p_temp[i].id + 1 << ' ';
        }
        cout << endl;
    }
    return 0;
}