#pragma comment(linker, "/Stack:512000000")
#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <cstdio>
#include <cmath>
#include <vector>
#include <string>
#include <cstring>
#include <map>
#include <set>
#include <algorithm>
#include <unordered_map>
#include <unordered_set>

using namespace std;

typedef long long LL;
typedef double dbl;

struct Point {
  dbl x, y;
  int id;

  bool operator<(const Point &p) const {
    return id < p.id;
  }
};

Point operator-(const Point &to, const Point &from) {
  return Point{ to.x - from.x, to.y - from.y, -1 };
}

typedef vector<Point> polygon;

dbl CrossProduct(const Point &a, const Point &b) {
  return a.x * b.y - a.y * b.x;
}

struct Comp {
  Point pivot;

  bool operator()(const Point &a, const Point &b) {
    Point va = a - pivot;
    Point vb = b - pivot;
    
    dbl cross = CrossProduct(va, vb);

    if (fabs(cross) < 1e-9) return hypot(va.x, va.y) < hypot(vb.x, vb.y);
    return cross > -1e-9;
  }
};

polygon P;

polygon convexHull(polygon poly) {
  Comp cmp;
  cmp.pivot = Point{ 1e9, 1e9, -1 };
  for (const auto &point : poly) {
    if (pair<dbl, dbl>{cmp.pivot.x, cmp.pivot.y} > pair<dbl, dbl>{point.x, point.y}) cmp.pivot = point;
  }

  sort(poly.begin(), poly.end(), cmp);

  polygon ans;

  for (const auto &p : poly) {
    while (ans.size() >= 2 && CrossProduct(p - ans.back(), ans[ans.size() - 2] - ans.back()) < -1e-9) ans.pop_back();
    ans.push_back(p);
  }
  return ans;
}

bool ask(const polygon &p) {
  if (p.size() < 3) return 0;
  cout << "? " << p.size() << ' ';
  for (const auto &pt : p) {
    cout << pt.id + 1 << ' ';
  }
  cout << endl;
  string resp;
  cin >> resp;
  return resp[0] == 'Y';
}
bool ask(const set<int> &p) {
  if (p.size() < 3) return 0;
  cout << "? " << p.size() << ' ';
  for (const auto &pt : p) {
    cout << pt + 1 << ' ';
  }
  cout << endl;
  string resp;
  cin >> resp;
  return resp[0] == 'Y';
}

bool answer(const polygon &p) {
  cout << "! " << p.size() << ' ';
  for (const auto &pt : p) {
    cout << pt.id + 1 << ' ';
  }
  cout << endl;
  return 1;
}

void findTiger() {
  polygon poly = P;

  int lockedcnt = 0;
  vector<int> locked(poly.size());
  vector<int> todel(poly.size());

  while (lockedcnt != poly.size()) {
    polygon ch = convexHull(poly);


    for (int i = 0; i < ch.size(); ++i) {
      if (locked[ch[i].id]) continue;

      polygon kek;
      for (int j = 0; j < poly.size(); ++j) {
        if (poly[j].id == ch[i].id) continue;
        kek.push_back(poly[j]);
      }

      if (ask(convexHull(kek))) {
        todel[ch[i].id] = 1;
        break;
      } else {
        locked[ch[i].id] = 1;
        lockedcnt++;
      }

    }

    polygon next;
    for (const auto &t : poly) {
      if (todel[t.id]) continue;
      next.push_back(t);
    }
    poly.swap(next);
  }
  answer(convexHull(poly));
}

int solve() {
  srand(2017);
  int n;
  cin >> n;

  P.resize(n);

  for (int i = 0; i < n; ++i) {
    cin >> P[i].x >> P[i].y;
    P[i].id = i;
  }
  
  int q;
  cin >> q;

  while (q--) findTiger();

  return 0;
}

int main() {
  solve();

  return 0;
}